from __future__ import print_function
__author__ = 'marco'

import numpy as np
import matplotlib.pyplot as plt
import hierarchicalDBSCAN.Clash.toolbox as toolbox
from hierarchicalDBSCAN.Robustness_tests import generate_dataset
import prettyplotlib as ppl
from scipy.spatial.distance import cdist

def get_circle_hull(ps, initial_hull_edges=None, termination_criterion='delauney', adaptive_termination=True,
                    stop_at_disjoin=False, counter=None):

    if len(ps) < 3:
        return [], counter

    current_hull_edges = initial_hull_edges if initial_hull_edges is not None else toolbox.get_convex_hull(ps)

    longest_edge, longest_edge_length = toolbox.get_longest_edge(ps, current_hull_edges, 0)
    counter = 0 if counter is None else counter

    terminate = False
    while terminate == False:

        old_hull_edges = current_hull_edges
        current_hull_edges = get_circle_hull__step(ps, current_hull_edges)
        terminate = True

    return [current_hull_edges], counter






def get_circle_hull__step(ps, current_hull_edges, plot_step=False, verbose=False):

    skip_counter = 0

    longest_edge, longest_edge_length = toolbox.get_longest_edge(ps, current_hull_edges, rank=skip_counter)

    return current_hull_edges


base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]*100


def draw_labeling(data, labeling, true_labeling=False):
    labeling = np.asarray(labeling)
    if len(set(labeling)) > 1:
        colors = {label: [t/255 for t in base_colors[(i*3)]] for i, label in enumerate(set(labeling))}
        for label in set(labeling):
            filtered_dataset = data[np.where(labeling==label)[0].tolist(), ]
            if label == -1 and not true_labeling:
                ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                            color = [0.9, 0.9, 0.9], linewidth=1, marker='x', edgecolor='black')
            else:
                #filtered_dataset = dataset[np.where(labeling == label)[0].tolist(),:]
                ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                            color=colors[label])
    else:
        ppl.scatter(plt.gca(), data[:, 0], data[:, 1], label=str(1),
                    linewidth=1, edgecolor='black',
                    color=[t/255 for t in base_colors[0]])


if __name__ == "__main__":
    print("test")
    dataset, labels_true = generate_dataset("moons", {"n_samples": 300, "noise": 0.075, "random_state": None})
    dataset = dataset[labels_true==0,:]
    labels_true = labels_true[labels_true==0]

    fig = plt.figure(figsize=(16, 4))
    plt.subplot(1, 4, 1)
    draw_labeling(dataset, labels_true, True)
    plt.title("BaseShape", size=20)
    fig.tight_layout()
    fig.subplots_adjust(hspace=0.3)


    plt.subplot(1, 3, 2)
    plt.axis('equal')
    initial_hull_edges=None
    current_hull_edges = initial_hull_edges if initial_hull_edges is not None else toolbox.get_convex_hull(dataset)

    current_hull_edges_with_last_edge = current_hull_edges[:]
    current_hull_edges_with_last_edge.append((current_hull_edges[-1][1], current_hull_edges[0][0]))
    current_hull_edges_with_last_edge = np.asarray(current_hull_edges_with_last_edge)
    plt.plot(dataset[current_hull_edges_with_last_edge[:, 0]][:, 0], dataset[current_hull_edges_with_last_edge[:, 0]][:, 1], 'b-')

    ppl.scatter(plt.gca(), dataset[:, 0], dataset[:, 1], label=str(0),
                color=[t / 255 for t in base_colors[0]], linewidth=1, edgecolor='black')

    terminate = False
    while terminate == False:
        longest_edge, longest_edge_length = toolbox.get_longest_edge(dataset, current_hull_edges, rank=0)
        mid = np.mean(dataset[longest_edge,:], axis=0)
        ppl.scatter(plt.gca(), mid[0], mid[1], label=str(0),
                    color=[t / 255 for t in base_colors[3]], linewidth=1, edgecolor='black')

        Y = cdist(np.array([mid]), dataset, 'euclidean')
        min_el = np.argmin(Y)
        nearest_point = dataset[min_el]
        ppl.scatter(plt.gca(), nearest_point[0], nearest_point[1], label=str(0),
                    color=[t / 255 for t in base_colors[3]], linewidth=1, edgecolor='black')

        if min_el in longest_edge:
            print("no close points, remove point from hull")
            idx = current_hull_edges.index(longest_edge)
            pt1 = longest_edge[0]
            pt2 = longest_edge[1]

            edge1 = current_hull_edges[(idx-1)%len(current_hull_edges)]
            dist1 = np.linalg.norm(dataset[pt1, :] - dataset[edge1[0], :])

            edge2 = current_hull_edges[(idx+1)%len(current_hull_edges)]
            dist2 = np.linalg.norm(dataset[pt2, :] - dataset[edge2[1], :])

            current_hull_edges.pop(idx)
            if dist1< dist2:
                current_hull_edges.pop((idx-1)%len(current_hull_edges))
                current_hull_edges.insert(idx-1, (pt1, edge2[1]))
            else:
                current_hull_edges.pop(idx % len(current_hull_edges))
                current_hull_edges.insert(idx, (edge1[0], pt2))

        else:
            print("add edges to hull")
            idx = current_hull_edges.index(longest_edge)
            current_hull_edges.pop(idx)
            current_hull_edges.insert(idx, (min_el, longest_edge[1]))
            current_hull_edges.insert(idx, (longest_edge[0], min_el))


        plt.subplot(1, 3, 3)
        current_hull_edges_with_last_edge = current_hull_edges[:]
        current_hull_edges_with_last_edge.append((current_hull_edges[-1][1], current_hull_edges[0][0]))
        current_hull_edges_with_last_edge = np.asarray(current_hull_edges_with_last_edge)

        plt.plot(dataset[current_hull_edges_with_last_edge[:, 0]][:, 0],
                 dataset[current_hull_edges_with_last_edge[:, 0]][:, 1], 'b-')
        plt.axis('equal')
        ppl.scatter(plt.gca(), dataset[:, 0], dataset[:, 1], label=str(0),
                    color=[t / 255 for t in base_colors[0]], linewidth=1, edgecolor='black')
        plt.show()
