__author__ = 'Alex'
import numpy as np
import matplotlib.pyplot as plt
import skfuzzy as fuzz
from sklearn import datasets
import prettyplotlib as ppl

np.random.seed(2)
colors = ['b', 'orange', 'g', 'r', 'c', 'm', 'y', 'k', 'Brown', 'ForestGreen']
iris = datasets.load_iris()
dataset = iris.data[:, 2:4]  # we only take the first two features.
Y = iris.target

base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]*10

# Define three cluster centers
centers = [[6, 2],
           [4, 1],
           [1, 0]]

# Define three cluster sigmas in x and y, respectively
sigmas = [[0.8, 0.3],
          [0.3, 0.5],
          [1.1, 0.7]]

# Generate test data
X = dataset
xpts = X[:, 0]
ypts = X[:, 1]
labels = Y

fig = plt.figure(figsize=(8, 3))


plt.subplot(1, 2, 1)
labels = Y
colors = {label: [t/255 for t in base_colors[(i*3)]] for i, label in enumerate(set(labels))}
for label in set(labels):
    #print("label", label)
    filtered_dataset = dataset[np.where(labels == label), ][0]
    ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                color=colors[label], linewidth=1, edgecolor='black', alpha=1)
plt.xlabel('Petal length')
plt.ylabel('Petal width')

plt.subplot(1, 2, 2)
alldata = np.vstack((xpts, ypts))
cntr, u, u0, d, jm, p, fpc = fuzz.cluster.cmeans(
    alldata, 3, 2, error=0.005, maxiter=10, init=None)


labels = np.argmax(u, axis=0)
print(labels)

colors = {label: [t/255 for t in base_colors[i*3]] for i, label in enumerate(set(Y))}
print(u)
for label in set(labels):
    filtered_dataset = dataset[np.where(labels == label), ][0]
    alphas = list(np.max(u[:, np.where(labels == label)], axis=0)[0])
    for i, point in enumerate(filtered_dataset):
        #color = colors[label]+
        ppl.scatter(plt.gca(), point[0], point[1], label=str(label), linewidth=1, edgecolor='black',
                    color=colors[label], alpha=alphas[i]**3)
plt.xlabel('Petal length')
plt.ylabel('Petal width')


fig.tight_layout()
fig.savefig('../../latex script/images/iris-example-fuzzy.png')
plt.show()