__author__ = 'Alex'

import numpy as np
from sklearn.cluster import DBSCAN
import matplotlib.pyplot as plt
from sklearn.datasets import make_moons
from Robustness_tests import generate_dataset
from Datasets.DatasetLoader import *
import colorsys
import random

def dbscan(dataset, eps=0.3, min_samples=10):
    db = DBSCAN(eps=eps, min_samples=min_samples).fit(dataset)
    core_samples_mask = np.zeros_like(db.labels_, dtype=bool)
    core_samples_mask[db.core_sample_indices_] = True
    return db.labels_, core_samples_mask

def plot(dataset, labels, name, values):
    max_labels = len(set(labels))
    hsv_tuples = [(x * 1.0 / max_labels, 0.7, 0.9) for x in range(max_labels)]
    rgb_tuples = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    random.shuffle(rgb_tuples)
    #plt.gca().axis("equal")
    plt.title("{} (eps={}, minPts={})".format(name, values[0], values[1]))
    for label in set(labels):
        #print("label", label)
        filtered_dataset = dataset[np.where(labels == label), ][0]

        if label < 0:
            # plot noise
            plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c='black', s=50, marker='x')
            test =0
        else:
            # plot points in a cluster
            plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c=rgb_tuples[label], s=50)


def example_aggregation_parameters():
    fig = plt.figure(figsize=(11, 15))

    np.random.seed(17041991)
    #dataset, labels_true = aggregation()
    #dataset, labels_true = spiral()
    #dataset, labels_true = flame()
    dataset, labels_true = generate_dataset("moons", {"n_samples": 100, "noise": 0.075, "random_state":None})
    #dataset, labels_true = generate_dataset("blobs", {"n_samples": 100, "n_features": 1000, "random_state":None})


    plt.subplot(3, 2, 1)
    np.random.seed(17041991)
    #values = [1.209338662244726, 5]
    values = [3.020347662107782, 6]
    #values = [0.91241437954472293, 5]
    #values = [45.484507234640844, 5]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Aggregation", values)

    plt.subplot(3, 2, 2)
    np.random.seed(17041991)
    #values = [1.209338662244726, 2]
    values = [3.020347662107782, 3]
    #values = [0.91241437954472293, 6]
    #values = [45.484507234640844, 19]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Moons", values)

    plt.subplot(3, 2, 3)
    np.random.seed(17041991)
    #values = [1.0977249200049743, 2]
    #values = [1.0511898020816568, 2]
    #values = [0.9924716620640246, 6]
    values = [46.414670265511319, 19]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Blobs-1000D", values)

    plt.subplot(3, 2, 4)
    np.random.seed(17041991)
    #values = [1.0977249200049743, 1]
    #values = [1.0511898020816568, 1]
    values = [46.414670265511319, 27]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Spirals", values)

    plt.subplot(3, 2, 5)
    np.random.seed(17041991)
    #values = [1.4508618128546971, 1]
    #values = [1.1067971810589365, 1]
    values = [47.132403543606983, 27]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "R15", values)

    plt.subplot(3, 2, 5)
    np.random.seed(17041991)
    #values = [1.4508618128546971, 1]
    #values = [1.1067971810589365, 1]
    values = [47.132403543606983, 31]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "R15", values)

    plt.subplot(3, 2, 6)
    np.random.seed(17041991)
    #values = [1.4508618128546971, 1]
    #values = [1.4508618128546971, 1]
    values = [47.27658174688051, 31]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "R15", values)

    plt.show()


if __name__ == "__main__":
    ## example datasets
    #dataset, labels = trex()
    #dataset, labels_true = aggregation()
    #dataset, labels_true = compound()
    #dataset, labels_true = r15()
    #dataset, labels_true = d31()
    #dataset, labels_true = jain()
    #dataset, labels_true = pathbased()
    #dataset = t48k(size=4000)
    #dataset, labels_true = spiral()
    #dataset, labels_true = flame()

    example_aggregation_parameters()
