__author__ = 'Alex'

import numpy as np
from sklearn.cluster import DBSCAN
import matplotlib.pyplot as plt
import colorsys
import prettyplotlib as ppl

from hierarchicalDBSCAN.Datasets.DatasetLoader import standard
from sklearn.datasets import make_moons

def dbscan(dataset, eps=0.3, min_samples=10):
    db = DBSCAN(eps=eps, min_samples=min_samples).fit(dataset)
    core_samples_mask = np.zeros_like(db.labels_, dtype=bool)
    core_samples_mask[db.core_sample_indices_] = True
    return db.labels_, core_samples_mask

def example_dbscan_parameters_paper():
    np.random.seed(17041991)

    # build dataset
    noisy_moons, labels = make_moons(n_samples=250, noise=.05)
    noisy_moons = noisy_moons[np.where(labels == 1), ][0]
    mu_x = 1.0
    mu_y = 0.45
    sigma = 0.20
    x = np.array(np.random.normal(mu_x, sigma, 300))
    y = np.array(np.random.normal(mu_y, sigma, 300))
    gaussian_blob = np.matrix(list(zip(x,y)))

    dataset = np.append(noisy_moons, gaussian_blob, axis=0)

    labels = [0] * len(noisy_moons) + [1]*len(gaussian_blob)

    #dbscan(dataset=)
    value_combinations = [(0.1, 8), (0.13, 8), (0.1, 5), (0.13, 5)]
    fig = plt.figure(figsize=(15, 7))
    labelings = []
    max_labels = 0
    for idx_values in range(len(value_combinations)):
        values = value_combinations[idx_values]
        labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
        labelings.append(labels)
        if len(set(labelings[-1])) > max_labels:
            max_labels = len(set(labelings[-1]))

    hsv_tuples = [(x * 1.0 / max_labels, 0.7, 0.9) for x in range(max_labels)]
    rgb_tuples = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    #print(labelings)
    for idx_value in range(len(value_combinations)):
        #print("idx_values", idx_value)
        labels = labelings[idx_value]
        #print("labels", labels)
        plt.subplot(2, 2, idx_value+1)
        values = value_combinations[idx_value]
        plt.title("DBSCAN (eps={}, minPts={})".format(values[0], values[1]))
        for label in set(labels):
            #print("label", label)
            filtered_dataset = dataset[np.where(labels == label), ][0]

            if label == -1:
                # plot noise
                plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c='black', s=50, marker='x')
                test =0
            else:
                # plot points in a cluster
                plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c=rgb_tuples[label], s=50)
        plt.gca().axis("equal")

        # plot points outside a cluster
    plt.show()

def example_dbscan_parameters_thesis():
    np.random.seed(17041991)

    # load dataset
    dataset, labels_true = standard()

    #dbscan(dataset=)
    value_combinations = [(0.08, 10), (0.1, 10), (0.08, 5), (0.1, 5)]
    labelings = []
    for idx_values in range(len(value_combinations)):
        values = value_combinations[idx_values]
        labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
        labelings.append(labels)


    fig = plt.figure(figsize=(8, 6))
    base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
                   (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
                   (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]
    #fig, ax = plt.subplots(2,2)

    #print(labelings)
    for idx_value in range(len(value_combinations)):

        labels = labelings[idx_value]
        plt.subplot(2, 2, idx_value+1)
        values = value_combinations[idx_value]
        plt.title(r'$DBSCAN (\epsilon= %s, min_{Pts}= %s)$'%(values[0], values[1]))
        #plt.title("DBSCAN (eps={}, minPts={})".format())

        colors = {label: [t/255 for t in base_colors[i]] for i, label in enumerate(set(labels))}

        for label in set(labels):
            #print("label", label)
            filtered_dataset = dataset[np.where(labels == label), ][0]

            if label == -1:
                # plot noise
                plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c='black', label=str(-1), s=20, marker='x')
            else:
                # plot points in a cluster
                ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                            color=colors[label], linewidth=1, edgecolor='black', alpha=0.5)
        plt.gca().axis("equal")
    fig.tight_layout()
    fig.savefig('../../latex script/images/dbs-varying-parameters.png')
    plt.show()

if __name__ == "__main__":
    #example_dbscan_parameters_paper()
    example_dbscan_parameters_thesis()