__author__ = 'Alex'
import prettyplotlib as ppl

import matplotlib.pyplot as plt
from hierarchicalDBSCAN.Datasets.DatasetLoader import *
from sklearn.metrics import pairwise_distances
from hierarchicalDBSCAN.ClusteringAlgorithms.minPtsGraphDBScan import *
from hierarchicalDBSCAN.ClusterValidation.InternalValidation import get_modified_silhouette_score
from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes
from mpl_toolkits.axes_grid1.inset_locator import mark_inset
from matplotlib.ticker import MaxNLocator

np.random.seed(0)
dataset, labels_true = generate_dataset("moons", {"n_samples": 100, "noise": 0.075, "random_state":None})
#dataset, labels_true = standard()

base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]

labels = ["unfiltered", "filtered"]
colors = {label: [t/255 for t in base_colors[(i+1)*3]] for i, label in enumerate(labels)}
statistics=[]

dbs = minPtsGraphDBScan(minPts=3)
dist_mat = pairwise_distances(dataset)
dbs.fit(dataset)


i = 0
print("time to weight")
heights = []
scores = []
label_gen = dbs.cut_height_label_gen(detailed_hierarchy=True)
for (height, label, clusterheight) in label_gen:
    print(i)
    i+=1
    heights.append(height)
    scores.append(get_modified_silhouette_score(dist_mat, label, 3))
statistics.append((heights[2:-2], scores[2:-2]))


print("time to weight again")
heights = []
scores = []
label_gen = dbs.cut_height_label_gen(detailed_hierarchy=False)
for (height, label, clusterheight) in label_gen:
    print(i)
    i+=1
    heights.append(height)
    scores.append(get_modified_silhouette_score(dist_mat, label, 3))
statistics.append((heights[:-1], scores[:-1]))


fig = plt.figure(figsize=(7, 2.5))
art = []
for i, label in enumerate(labels):
    ppl.plot(plt.gca(), statistics[i][0],  statistics[i][1], color=colors[label], lw=2.5, label=str(label), alpha=0.5)
ax = plt.gca()

lgd = ppl.legend(loc=9, bbox_to_anchor=(0.5, -0.2), ncol=2, frameon=False)
art.append(lgd)
plt.xlabel('cut height')
plt.ylabel('density-based\nsilhouette score')


axins = zoomed_inset_axes(plt.gca(),3 , loc=4) # zoom = 6
for i, label in enumerate(labels):
    ppl.plot(plt.gca(), statistics[i][0],  statistics[i][1], color=colors[label], lw=2.5, label=str(label), alpha=0.5)

#axins.plot(xbt_raw['depth'], xbt_raw['temp'], **raw)

axins.axis([0.12, 0.14, 0.65, 0.85])
axins.spines["top"].set_visible(True)
axins.spines["right"].set_visible(True)
mark_inset(ax, axins, loc1=1, loc2=3, fc="none", ec="0.5")
#axins.xaxis.set_major_locator(MaxNLocator(nbins=1, prune='lower'))

plt.xticks(visible=False)
plt.yticks(visible=False)

#fig.tight_layout()
plt.show()

fig.savefig('../../latex script/images/hierarchy-filtering.png', additional_artists=art, bbox_inches="tight")
