__author__ = 'Alex'

from sklearn import datasets
import matplotlib.pyplot as plt
#import guiqwt.pyplot as plt

import scipy.spatial
import scipy.spatial.distance
import numpy as np
import matplotlib.path as mplPath
import math
import matplotlib.patches as patches
import itertools as IT
import logging

def drange(start, stop, step):
    r = start
    while r < stop:
        yield r
        r += step


def grid_based_test(dataset, nr_of_grid_cells=10):
    dim = dataset.ndim
    minmax = []
    grid_boarders = []
    for i in range(dim):
        minmax.append((np.amin(dataset[:,i]), np.amax(dataset[:,i])))
        dim_range = minmax[-1][1] - minmax[-1][0]
        dim_unit = dim_range/nr_of_grid_cells
        grid_boarders.append(np.arange(minmax[-1][0]-dim_unit/2, minmax[-1][1]+dim_unit, dim_unit))


    #generate random samples



def triangulation_test(dataset):
    tri = scipy.spatial.Delaunay(dataset)
    print()
    plt.triplot(dataset[:, 0], dataset[:, 1], tri.simplices.copy())
    plt.plot(dataset[:, 0], dataset[:, 1], 'o')
    plt.show()




def voronoi_test(dataset):
    vor = scipy.spatial.Voronoi(dataset)
    scipy.spatial.voronoi_plot_2d(vor)
    plt.show()


def generate_dataset(name, params):
    if name == "moons":
        return datasets.make_moons(**params)
    elif name == "circles":
        return datasets.make_circles(**params)
    elif name == "blobs":
        return datasets.make_blobs(**params)
    elif name == "swiss":
        return dataset.make_swiss_roll(**params)
    elif name == "boston":
        datasets.load_boston()[0]
    elif name == "diabetes":
        datasets.load_diabetes()[0]
    elif name == "digits":
        datasets.load_digits()[0]
    else:
        raise Exception('The dataset', name, 'could not be generated! Available data sets are "moons", "circles", "blobs", "swiss", "boston", "diabetes", "digits"')


def generate_uniform_noise(n, polygon, plot=False):
    points = []

    #calculate bounding boxes
    range_x = (np.amin(polygon[:,0]),np.amax(polygon[:,0]))
    range_y = (np.amin(polygon[:,1]), np.amax(polygon[:,1]))
    logging.debug("ranges: {0}, {1}".format(range_x, range_y))
    bbPath = mplPath.Path(polygon)

    #drawing random points and check for containment
    while len(points) < n:
        new_point_x = np.random.uniform(range_x[0], range_x[1])
        new_point_y = np.random.uniform(range_y[0], range_y[1])
        new_point = np.array([new_point_x, new_point_y])
        if bbPath.contains_point(new_point):
            points.append(new_point)

    #plotting
    if plot:
        ax = plt.gca()
        patch = patches.PathPatch(bbPath, facecolor='orange', lw=2)
        ax.add_patch(patch)
        ax.set_xlim(range_x[0]-1,range_x[1]+1)
        ax.set_ylim(range_y[0]-1,range_y[1]+1)
        for (x,y) in points:
            plt.plot(x,y,'+')
    return points


def generate_gaussian_noise(n, polygon, plot):
    points = []

    #calculate bounding boxes
    range_x = (np.amin(polygon[:,0]),np.amax(polygon[:,0]))
    range_y = (np.amin(polygon[:,1]), np.amax(polygon[:,1]))
    logging.debug("ranges: {0}, {1}".format(range_x, range_y))
    bbPath = mplPath.Path(polygon)
    centroid = centroid_of_polygon(polygon)

    #drawing random points and check for containment
    while len(points) < n:
        #todo estimate standard deviation. currently = 0.3, maybe use 0.3 times distance from centroid to boarder
        new_point_x = np.random.normal(centroid[0], 0.3)
        new_point_y = np.random.normal(centroid[1], 0.3)
        new_point = np.array([new_point_x, new_point_y])
        if bbPath.contains_point(new_point):
            points.append(new_point)

    #plotting
    if plot:
        ax = plt.gca()
        patch = patches.PathPatch(bbPath, facecolor='orange', lw=2)
        ax.add_patch(patch)
        ax.set_xlim(range_x[0]-1,range_x[1]+1)
        ax.set_ylim(range_y[0]-1,range_y[1]+1)
        for (x,y) in points:
            plt.plot(x,y,'+')
    return points

def generate_ringed_noise(n, polygon, plot):
    points = []

    #calculate bounding boxes
    range_x = (np.amin(polygon[:,0]),np.amax(polygon[:,0]))
    range_y = (np.amin(polygon[:,1]), np.amax(polygon[:,1]))
    logging.debug("ranges: {0}, {1}".format(range_x, range_y))
    bbPath = mplPath.Path(polygon)
    centroid = centroid_of_polygon(polygon)
    max_distance = 0
    for point in polygon:
        distance = scipy.spatial.distance.euclidean(point, centroid)
        if distance > max_distance:
            max_distance = distance

    #drawing random points and check for containment
    while len(points) < n:
        new_point_x = np.random.uniform(range_x[0], range_x[1])
        new_point_y = np.random.uniform(range_y[0], range_y[1])
        new_point = np.array([new_point_x, new_point_y])
        #acceptance rate:
        #todo calculate acceptance rate
        acceptance_rate = scipy.spatial.distance.euclidean(new_point, centroid) / (max_distance*2)
        if np.random.uniform(0,1)<acceptance_rate:
            if bbPath.contains_point(new_point):
                points.append(new_point)

    #plotting
    if plot:
        ax = plt.gca()
        patch = patches.PathPatch(bbPath, facecolor='orange', lw=2)
        ax.add_patch(patch)
        ax.set_xlim(range_x[0]-1,range_x[1]+1)
        ax.set_ylim(range_y[0]-1,range_y[1]+1)
        for (x,y) in points:
            plt.plot(x,y,'+')
    return points


def area_of_polygon(x, y):
    """Calculates the signed area of an arbitrary polygon given its verticies
    http://stackoverflow.com/a/4682656/190597 (Joe Kington)
    http://softsurfer.com/Archive/algorithm_0101/algorithm_0101.htm#2D%20Polygons
    """
    area = 0.0
    for i in range(-1, len(x) - 1):
        area += x[i] * (y[i + 1] - y[i - 1])
    return area / 2.0

def centroid_of_polygon(points):
    """
    http://stackoverflow.com/a/14115494/190597 (mgamba)
    """
    area = area_of_polygon(*zip(*points))
    result_x = 0
    result_y = 0
    N = len(points)
    points = IT.cycle(points)
    x1, y1 = next(points)
    for i in range(N):
        x0, y0 = x1, y1
        x1, y1 = next(points)
        cross = (x0 * y1) - (x1 * y0)
        result_x += (x0 + x1) * cross
        result_y += (y0 + y1) * cross
    result_x /= (area * 6.0)
    result_y /= (area * 6.0)
    return (result_x, result_y)


def generate_noise_demo(n, polygon):
    fig = plt.figure()

    plt.subplot(1, 3, 1)
    generate_uniform_noise(n, polygon, True)

    plt.subplot(1, 3, 2)
    generate_gaussian_noise(n, polygon, True)

    plt.subplot(1, 3, 3)
    generate_ringed_noise(n, polygon, True)

    plt.show()



if __name__ == "__main__":
    #generate_dataset("moons", n_samples=100, shuffle=True, noise=None, random_state=None)
    dataset = generate_dataset("blobs", {"n_samples":100, "random_state":None})
    #dataset = generate_dataset("moons", {"n_samples":100, "noise": None, "random_state":None})

    voronoi_test(dataset)
    #triangulation_test(dataset)
    #grid_based_test(dataset)


    polygon = np.array([[1, 1],
                         [3, 1],
                         [2, math.sqrt(3)],
                        [1,1]])
    #generate_noise_demo(1000, polygon)

    #generate_uniform_noise(1000, polygon)
    #print(centroid_of_polygon(polygon))

    polygon = np.array([[1,1],
                        [1,3],
                        [3,3],
                        [3,1],
                        [1,1]])
    #generate_noise_demo(1000, polygon)
    #generate_uniform_noise(1000, polygon)
